@import url(master.css);

h1 {
    font-size: 400%;
    font-weight: 700;
    margin-top: 0.1em;
    margin-bottom: 0.1em;
    border-bottom-style: solid;
    border-bottom-color: #884000;
    border-bottom-width: 0.1em;
}

p {
    text-align: left;
}

body {
    margin-bottom: 3em;
}

@media only screen and (min-width: 600px) {
    h1 {
        font-size: 800%;
    }

    .download-links {
        margin-bottom: 3ex;
        width: 100%;
        text-align: right;
    }

    .download-links ul {
        font-size: 0;
    }

    .download-links ul li {
        font-size: 1rem;
        font-family: sans-serif;
        list-style: none;
        display: inline;
    }

    .download-links ul li p {
        display: inline;
    }

    .download-links ul li:not(:last-child) {
        margin-right: 0.3em;
    }

    .download-links ul li:not(:first-child):before {
        margin-right: 0.3em;
        content: "|";
    }
}
