@tailwind base;

@tailwind components;

@tailwind utilities;


.light {
    color: #B58000;
}

.dark {
    color: #804000;
}


.document {
    @apply font-serif;
}

.document, .prev-next {
    @apply mx-auto;
    width: 100%;
}

@media (min-width: 576px) { 
    .document, .prev-next {
        max-width: 576px;
    }
}

@media (min-width: 768px) { 
    .document, .prev-next {
        max-width: 768px;
    }
}

@media (max-width: 576px) {
    html { 
        font-size: 12pt;
    }
}

.document {
    @apply mt-2;
}

a {
    @apply no-underline light dark;
}

a:hover {
    @apply underline light;
}

p {
    @apply mt-2 mb-2;
    hyphens: auto;
}

blockquote {
    @apply mt-4 mb-4 ml-8;
}

ul {
    @apply mt-2 mb-2 ml-8 list-disc;
}

ol {
    @apply mt-2 mb-2 ml-8 list-decimal;
}

li {
    @apply mt-1;
}

img {
    @apply border-none;
}

h1, h2, h3, h1 a, h2 a, h3 a {
    @apply font-sans light;
}

.search a {
    @apply font-sans;
}

h1 {
    @apply mt-16 mb-2 text-4xl;
}

h2 {
    @apply mt-12 mb-2 text-2xl;
}

h3 {
    @apply mt-8 mb-2 text-lg;
}

a.headerlink {
    @apply invisible no-underline;
}

h1:hover > a.headerlink,
h2:hover > a.headerlink,
h3:hover > a.headerlink,
h4:hover > a.headerlink,
h5:hover > a.headerlink,
h6:hover > a.headerlink,
dt:hover > a.headerlink,
caption:hover > a.headerlink,
p.caption:hover > a.headerlink,
div.code-block-caption:hover > a.headerlink {
    @apply visible dark;
}

.contents, .toctree-wrapper {
    @apply mt-8 mb-8 ml-8 font-sans dark;
}

.contents ul, .toctree-wrapper ul {
    @apply light;
}

.contents .topic-title {
    @apply hidden;
}

dl, dt {
    @apply mt-4;
}

dd {
    @apply ml-8;
}

code, pre, tt {
    @apply font-mono text-base;
    hyphens: none;
}

code {
    @apply whitespace-pre break-words;
}

code.descname {
    @apply font-bold light;
}

.function dt, .class dt {
    @apply font-mono;
}

.function dt em, .class dt em {
    @apply not-italic;
}

.versionmodified {
    @apply italic;
}

.sourcecode pre, .highlight pre {
    @apply mt-4 mb-4 ml-4 text-sm;
}

.field-list ul {
    margin: m-0;
}

.field-list li {
    @apply m-0;
}

.field-list th {
    @apply font-normal;
}

.field-list td strong {
    @apply font-normal font-mono;
}

.first {
    @apply mt-0 !important;
}

.left {
    @apply float-left;
}

.right {
    @apply float-right;
}

.middle {
    @apply inline-block text-center;
}

.relbar {
    @apply mt-4 ml-4 mr-4 text-sm;
}

form input {
    @apply border border-solid border-gray-500 pl-1 pr-1 text-sm rounded-sm;
}

.prev-next {
    @apply mt-12 text-center;
    clear: both;
    word-wrap: normal;
}

.footer {
    @apply mt-20 mb-4 text-xs text-center opacity-50;
    clear: both;
}

.admonition {
    border: 0.4ex solid;
    margin-top: 1ex !important;
    margin-bottom: 1ex !important;
    background: red;
}

.admonition p {
    margin: 1ex 1em;
    padding: 0;
}

.warning {
    border-color: #D04040;
}

.highlighted {
    background-color: #fbe54e;
}
